@@ -3,55 +3,53 @@
     @Author:            Maxence Larose
 
     @Creation Date:     01/2022
-    @Last modification: 01/2022
+    @Last modification: 03/2022
 
     @Description:       This file contains all factories that inherit from the BasePatientDataFactory class.
 """
 
 from typing import Dict, List, Optional
 
-from ....data_model import ImageAndSegmentationDataModel, PatientDataModel
-from ...image.dicom_reader import DicomReader
-from .base_patient_data_factory import BasePatientDataFactory
-from ...segmentation.segmentation_reader import SegmentationReader
+from dicom2hdf.data_readers.patient_data.factories.base_patient_data_factory import BasePatientDataFactory
+from dicom2hdf.data_model import ImageAndSegmentationDataModel, PatientDataModel
+from dicom2hdf.data_readers.image.dicom_reader import DicomReader
+from dicom2hdf.data_readers.segmentation.segmentation_reader import SegmentationReader
 
 
 class DefaultPatientDataFactory(BasePatientDataFactory):
     """
-    Class that defined the methods that are used to get the patient data. The default factory consists in obtaining all
+    Class that defines the methods that are used to get the patient data. The default factory consists in obtaining all
     the images without any segmentation.
     """
 
     def __init__(
             self,
-            path_to_images_folder: str,
-            path_to_segmentations_folder: Optional[str],
+            path_to_patient_folder: str,
+            paths_to_segmentations: Optional[List[str]],
             series_descriptions: Optional[Dict[str, List[str]]],
-            verbose: bool
+            erase_unused_dicom_files: bool = False
     ):
         """
-        Constructor of the class BasePatientDataFactory.
+        Constructor of the class DefaultPatientDataFactory.
 
         Parameters
         ----------
-        path_to_images_folder : str
+        path_to_patient_folder : str
             Path to the folder containing the patient's image files.
-        path_to_segmentations_folder : Optional[str]
-            Path to the folder containing the patient's segmentation files.
+        paths_to_segmentations : Optional[List[str]]
+            List of paths to the patient's segmentation files.
         series_descriptions : Optional[Dict[str, List[str]]]
             A dictionary that contains the series descriptions of the images that absolutely needs to be extracted from
             the patient's file. Keys are arbitrary names given to the images we want to add and values are lists of
-            series descriptions. The images associated with these series descriptions do not need to have a
-            corresponding segmentation. In fact, the whole point of adding a way to specify the series descriptions that
-            must be added to the dataset is to be able to add images without segmentation.
-        verbose : bool
-            True to log/print some information else False.
+            series descriptions.
+        erase_unused_dicom_files: bool = False
+            Whether to delete unused DICOM files or not. Use with caution.
         """
-        super(DefaultPatientDataFactory, self).__init__(
-            path_to_images_folder=path_to_images_folder,
-            path_to_segmentations_folder=path_to_segmentations_folder,
+        super().__init__(
+            path_to_patient_folder=path_to_patient_folder,
+            paths_to_segmentations=paths_to_segmentations,
             series_descriptions=series_descriptions,
-            verbose=verbose
+            erase_unused_dicom_files=erase_unused_dicom_files
         )
 
     def create_patient_data(self) -> PatientDataModel:
@@ -64,7 +62,7 @@ class DefaultPatientDataFactory(BasePatientDataFactory):
             Patient data.
         """
         patient_data = PatientDataModel(
-            patient_name=self.patient_name,
+            patient_id=self.patient_id,
             data=[ImageAndSegmentationDataModel(image=image) for image in self._images_data]
         )
         return patient_data
@@ -72,41 +70,39 @@ class DefaultPatientDataFactory(BasePatientDataFactory):
 
 class SegmentationPatientDataFactory(BasePatientDataFactory):
     """
-    Class that defined the methods that are used to get the patient data. The segmentation patient data factory consists
+    Class that defines the methods that are used to get the patient data. The segmentation patient data factory consists
     in obtaining the images that have the same serial uids as those contained in the file names of the given
     segmentations. The final dataset therefore contains both the segmentations and their corresponding images.
     """
 
     def __init__(
             self,
-            path_to_images_folder: str,
-            path_to_segmentations_folder: Optional[str],
+            path_to_patient_folder: str,
+            paths_to_segmentations: List[str],
             series_descriptions: Optional[Dict[str, List[str]]],
-            verbose: bool
+            erase_unused_dicom_files: bool = False
     ):
         """
-        Constructor of the class SegmentationOnlyPatientDataFactory.
+        Constructor of the class SegmentationPatientDataFactory.
 
         Parameters
         ----------
-        path_to_images_folder : str
+        path_to_patient_folder : str
             Path to the folder containing the patient's image files.
-        path_to_segmentations_folder : Optional[str]
-            Path to the folder containing the patient's segmentation files.
+        paths_to_segmentations : Optional[List[str]]
+            List of paths to the patient's segmentation files.
         series_descriptions : Optional[Dict[str, List[str]]]
             A dictionary that contains the series descriptions of the images that absolutely needs to be extracted from
             the patient's file. Keys are arbitrary names given to the images we want to add and values are lists of
-            series descriptions. The images associated with these series descriptions do not need to have a
-            corresponding segmentation. In fact, the whole point of adding a way to specify the series descriptions that
-            must be added to the dataset is to be able to add images without segmentation.
-        verbose : bool
-            True to log/print some information else False.
+            series descriptions.
+        erase_unused_dicom_files: bool = False
+            Whether to delete unused DICOM files or not. Use with caution.
         """
-        super(SegmentationPatientDataFactory, self).__init__(
-            path_to_images_folder=path_to_images_folder,
-            path_to_segmentations_folder=path_to_segmentations_folder,
+        super().__init__(
+            path_to_patient_folder=path_to_patient_folder,
+            paths_to_segmentations=paths_to_segmentations,
             series_descriptions=series_descriptions,
-            verbose=verbose
+            erase_unused_dicom_files=erase_unused_dicom_files
         )
 
     def create_patient_data(self) -> PatientDataModel:
@@ -120,20 +116,31 @@ class SegmentationPatientDataFactory(BasePatientDataFactory):
         """
         data = []
         for image in self._images_data:
+            image_added = False
+            segmentations = []
             for path_to_segmentation in self._paths_to_segmentations:
                 dicom_header = DicomReader.get_dicom_header(path_to_dicom=path_to_segmentation)
                 if image.dicom_header.SeriesInstanceUID == dicom_header.ReferencedSeriesSequence[0].SeriesInstanceUID:
-                    segmentation_reader = SegmentationReader(path_to_segmentation=path_to_segmentation)
-
-                    image_and_segmentation_data = ImageAndSegmentationDataModel(
+                    segmentation_reader = SegmentationReader(
                         image=image,
-                        segmentation=segmentation_reader.get_segmentation_data()
+                        path_to_segmentation=path_to_segmentation
                     )
 
-                    data.append(image_and_segmentation_data)
+                    segmentations.append(segmentation_reader.get_segmentation_data())
+
+            if segmentations:
+                image_and_segmentation_data = ImageAndSegmentationDataModel(
+                    image=image,
+                    segmentations=segmentations
+                )
+                data.append(image_and_segmentation_data)
+                image_added = True
+
+            if image_added is False and self._erase_unused_dicom_files:
+                self.erase_dicom_files(image)
 
         patient_data = PatientDataModel(
-            patient_name=self.patient_name,
+            patient_id=self.patient_id,
             data=data
         )
         return patient_data
@@ -141,41 +148,39 @@ class SegmentationPatientDataFactory(BasePatientDataFactory):
 
 class SeriesDescriptionPatientDataFactory(BasePatientDataFactory):
     """
-    Class that defined the methods that are used to get the patient data. The series description patient data factory
+    Class that defines the methods that are used to get the patient data. The series description patient data factory
     consists in obtaining only the images that have the given series descriptions. The final dataset therefore contains
     both the segmentations and their corresponding images.
     """
 
     def __init__(
             self,
-            path_to_images_folder: str,
-            path_to_segmentations_folder: Optional[str],
+            path_to_patient_folder: str,
+            paths_to_segmentations: Optional[List[str]],
             series_descriptions: Optional[Dict[str, List[str]]],
-            verbose: bool
+            erase_unused_dicom_files: bool = False
     ):
         """
-        Constructor of the class SegmentationOnlyPatientDataFactory.
+        Constructor of the class SeriesDescriptionPatientDataFactory.
 
         Parameters
         ----------
-        path_to_images_folder : str
+        path_to_patient_folder : str
             Path to the folder containing the patient's image files.
-        path_to_segmentations_folder : Optional[str]
-            Path to the folder containing the patient's segmentation files.
+        paths_to_segmentations : Optional[List[str]]
+            List of paths to the patient's segmentation files.
         series_descriptions : Optional[Dict[str, List[str]]]
             A dictionary that contains the series descriptions of the images that absolutely needs to be extracted from
             the patient's file. Keys are arbitrary names given to the images we want to add and values are lists of
-            series descriptions. The images associated with these series descriptions do not need to have a
-            corresponding segmentation. In fact, the whole point of adding a way to specify the series descriptions that
-            must be added to the dataset is to be able to add images without segmentation.
-        verbose : bool
-            True to log/print some information else False.
+            series descriptions.
+        erase_unused_dicom_files: bool = False
+            Whether to delete unused DICOM files or not. Use with caution.
         """
-        super(SeriesDescriptionPatientDataFactory, self).__init__(
-            path_to_images_folder=path_to_images_folder,
-            path_to_segmentations_folder=path_to_segmentations_folder,
+        super().__init__(
+            path_to_patient_folder=path_to_patient_folder,
+            paths_to_segmentations=paths_to_segmentations,
             series_descriptions=series_descriptions,
-            verbose=verbose
+            erase_unused_dicom_files=erase_unused_dicom_files
         )
         
     @property
@@ -201,54 +206,57 @@ class SeriesDescriptionPatientDataFactory(BasePatientDataFactory):
         """
         data = []
         for image_idx, image in enumerate(self._images_data):
+            image_added = False
             if image.dicom_header.SeriesDescription in self.flatten_series_descriptions:
                 image_data = ImageAndSegmentationDataModel(image=image)
                 data.append(image_data)
+                image_added = True
+
+            if image_added is False and self._erase_unused_dicom_files:
+                self.erase_dicom_files(image)
 
         patient_data = PatientDataModel(
-            patient_name=self.patient_name,
+            patient_id=self.patient_id,
             data=data
         )
         return patient_data
 
 
-class SegmentationAndSeriesDescriptionPatientDataFactory(BasePatientDataFactory):
+class SegAndSeriesPatientDataFactory(BasePatientDataFactory):
     """
-    Class that defined the methods that are used to get the patient data. The segmentation and series description
+    Class that defines the methods that are used to get the patient data. The segmentation and series description
     factory consists in obtaining the images that have the same serial uids as those contained in the file names of the
     given segmentations and the images that have the given series descriptions.
     """
 
     def __init__(
             self,
-            path_to_images_folder: str,
-            path_to_segmentations_folder: Optional[str],
+            path_to_patient_folder: str,
+            paths_to_segmentations: Optional[List[str]],
             series_descriptions: Optional[Dict[str, List[str]]],
-            verbose: bool
+            erase_unused_dicom_files: bool = False
     ):
         """
-        Constructor of the class SegmentationOnlyPatientDataFactory.
+        Constructor of the class SegAndSeriesPatientDataFactory.
 
         Parameters
         ----------
-        path_to_images_folder : str
+        path_to_patient_folder : str
             Path to the folder containing the patient's image files.
-        path_to_segmentations_folder : Optional[str]
-            Path to the folder containing the patient's segmentation files.
+        paths_to_segmentations : Optional[List[str]]
+            List of paths to the patient's segmentation files.
         series_descriptions : Optional[Dict[str, List[str]]]
             A dictionary that contains the series descriptions of the images that absolutely needs to be extracted from
             the patient's file. Keys are arbitrary names given to the images we want to add and values are lists of
-            series descriptions. The images associated with these series descriptions do not need to have a
-            corresponding segmentation. In fact, the whole point of adding a way to specify the series descriptions that
-            must be added to the dataset is to be able to add images without segmentation.
-        verbose : bool
-            True to log/print some information else False.
+            series descriptions.
+        erase_unused_dicom_files: bool = False
+            Whether to delete unused DICOM files or not. Use with caution.
         """
-        super(SegmentationAndSeriesDescriptionPatientDataFactory, self).__init__(
-            path_to_images_folder=path_to_images_folder,
-            path_to_segmentations_folder=path_to_segmentations_folder,
+        super().__init__(
+            path_to_patient_folder=path_to_patient_folder,
+            paths_to_segmentations=paths_to_segmentations,
             series_descriptions=series_descriptions,
-            verbose=verbose
+            erase_unused_dicom_files=erase_unused_dicom_files
         )
 
     @property
@@ -275,25 +283,45 @@ class SegmentationAndSeriesDescriptionPatientDataFactory(BasePatientDataFactory)
         data = []
         for image_idx, image in enumerate(self._images_data):
             image_added = False
-            for path_to_segmentation in self._paths_to_segmentations:
-                dicom_header = DicomReader.get_dicom_header(path_to_dicom=path_to_segmentation)
-                if image.dicom_header.SeriesInstanceUID == dicom_header.ReferencedSeriesSequence[0].SeriesInstanceUID:
-                    segmentation_reader = SegmentationReader(path_to_segmentation=path_to_segmentation)
-
-                    image_and_segmentation_data = ImageAndSegmentationDataModel(
-                        image=image,
-                        segmentation=segmentation_reader.get_segmentation_data()
-                    )
-                    data.append(image_and_segmentation_data)
-                    image_added = True
-
             series_description = image.dicom_header.SeriesDescription
-            if series_description in self.flatten_series_descriptions and image_added is False:
+
+            segmentations = []
+            for path_to_segmentation in self._paths_to_segmentations:
+                seg_header = DicomReader.get_dicom_header(path_to_dicom=path_to_segmentation)
+                if hasattr(seg_header, "ReferencedSeriesSequence"):
+                    reference_uid = seg_header.ReferencedSeriesSequence[0].SeriesInstanceUID
+                else:
+                    referenced_frame_of_reference_sequence = seg_header.ReferencedFrameOfReferenceSequence
+                    rt_referenced_study_sequence = referenced_frame_of_reference_sequence[0].RTReferencedStudySequence
+                    rt_referenced_series_sequence = rt_referenced_study_sequence[0].RTReferencedSeriesSequence
+                    reference_uid = rt_referenced_series_sequence[0].SeriesInstanceUID
+
+                if image.dicom_header.SeriesInstanceUID == reference_uid:
+                    segmentation_reader = SegmentationReader(
+                        image=image,
+                        path_to_segmentation=path_to_segmentation
+                    )
+
+                    segmentations.append(segmentation_reader.get_segmentation_data())
+
+            if segmentations:
+                image_and_segmentation_data = ImageAndSegmentationDataModel(
+                    image=image,
+                    segmentations=segmentations
+                )
+                data.append(image_and_segmentation_data)
+                image_added = True
+
+            if image_added is False and series_description in self.flatten_series_descriptions:
                 image_data = ImageAndSegmentationDataModel(image=image)
                 data.append(image_data)
+                image_added = True
+
+            if image_added is False and self._erase_unused_dicom_files:
+                self.erase_dicom_files(image)
 
         patient_data = PatientDataModel(
-            patient_name=self.patient_name,
+            patient_id=self.patient_id,
             data=data
         )
         return patient_data
