@@ -11,7 +11,7 @@
 
 from .segmentation_context import SegmentationContext
 from .factories.segmentation import Segmentation
-from ...data_model import SegmentationDataModel
+from ...data_model import ImageDataModel, SegmentationDataModel
 
 
 class SegmentationReader:
@@ -22,6 +22,7 @@ class SegmentationReader:
 
     def __init__(
             self,
+            image: ImageDataModel,
             path_to_segmentation: str,
     ):
         """
@@ -29,11 +30,30 @@ class SegmentationReader:
 
         Parameters
         ----------
+        image : ImageDataModel
+            A named tuple grouping the patient's dicom header, its medical image as a simpleITK image and a sequence of
+            the paths to each dicom contained in the series.
         path_to_segmentation : str
             The path to the segmentation file.
         """
+        self._image = image
         self._path_to_segmentation = path_to_segmentation
 
+    @property
+    def __segmentation_context_manager(self) -> SegmentationContext:
+        """
+        Creates a SegmentationContext object.
+
+        Returns
+        -------
+        segmentation_context : SegmentationContext
+            Segmentation context manager.
+        """
+        return SegmentationContext(
+            image=self._image,
+            path_to_segmentation=self._path_to_segmentation
+        )
+
     @property
     def __segmentation(self) -> Segmentation:
         """
@@ -44,11 +64,7 @@ class SegmentationReader:
         segmentation : Segmentation
             Segmentation.
         """
-        segmentation_context_manager = SegmentationContext(
-            path_to_segmentation=self._path_to_segmentation,
-        )
-
-        return segmentation_context_manager.create_segmentation()
+        return self.__segmentation_context_manager.create_segmentation()
 
     def get_segmentation_data(self) -> SegmentationDataModel:
         """
@@ -59,6 +75,9 @@ class SegmentationReader:
         segmentation_data : SegmentationDataModel
             A named tuple grouping the segmentation data.
         """
-        segmentation_data = SegmentationDataModel(simple_itk_label_maps=self.__segmentation.simple_itk_label_maps)
+        segmentation_data = SegmentationDataModel(
+            modality=self.__segmentation_context_manager.segmentation_strategy.modality,
+            simple_itk_label_maps=self.__segmentation.simple_itk_label_maps
+        )
 
         return segmentation_data
